use std::fmt;
use std::error::Error;
use std::collections::HashMap;

use serde::Deserialize;
use uuid::Uuid;
use chrono::{Datelike, Timelike, Utc};

pub struct SBatch {
    pub job_type: String,
    pub version: String,
    pub software: String,
    pub timestamp: String,
}

#[cfg(not(tarpaulin_include))]
impl SBatch {
    pub fn new(job_type: String, version: String, software: String) -> SBatch {
        let now = Utc::now();
        let (_is_ce, year) = now.year_ce();
        
        SBatch {
            job_type,
            version,
            software,
            timestamp: format!("{}{:02}{:02}T{:02}{:02}", year, now.month(), now.day(), now.hour(), now.minute()),
        }
    }
    
    pub fn get_file_path(&self) -> String {
        format!("{}/{}/{}/{}.sbatch", &self.software, &self.version, &self.job_type, &self.timestamp)
    }

    pub fn get_dir(&self) -> String {
        format!("{}/{}/{}", &self.software, &self.version, &self.job_type)
    }
}

#[derive(Deserialize, Clone, Debug, PartialEq, Eq)]
pub enum JobConfValueType {
    String,
    Boolean,
    Integer,
    Float,
    None
}

#[cfg(not(tarpaulin_include))]
impl fmt::Display for JobConfValueType {
    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
        match &self {
            JobConfValueType::String => write!( formatter, "string"),
            JobConfValueType::Boolean => write!( formatter, "bool"),
            JobConfValueType::Integer => write!( formatter, "int"),
            JobConfValueType::Float => write!( formatter, "float"),
            JobConfValueType::None => write!( formatter, "null"),
        }
    }
}

#[derive(Deserialize, Clone, Debug, PartialEq, Eq)]
pub struct JobConfCommandParam {
    pub name: String,
    #[serde(default = "default_value_type")]
    pub value_type: JobConfValueType,
    #[serde(default = "default_value")]
    pub default_value: Option<String>,
    #[serde(default = "default_comment")]
    pub comment: Option<String>,
    #[serde(default = "default_required")]
    pub required: bool,
    #[serde(default = "default_named")]
    pub named: bool,
    #[serde(default = "default_parameter_name")]
    pub parameter_name: Option<String>
}

#[cfg(not(tarpaulin_include))]
fn default_value_type() -> JobConfValueType {
    JobConfValueType::String
}

#[cfg(not(tarpaulin_include))]
fn default_value() -> Option<String> {
    None
}

#[cfg(not(tarpaulin_include))]
fn default_comment() -> Option<String> {
    None
}

#[cfg(not(tarpaulin_include))]
fn default_required() -> bool {
    false
}

#[cfg(not(tarpaulin_include))]
fn default_named() -> bool {
    false
}

#[cfg(not(tarpaulin_include))]
fn default_parameter_name() -> Option<String> {
    None
}

#[derive(Deserialize, Debug, Clone, PartialEq, Eq)]
pub struct JobConfCommand {
    #[serde(skip, default = "get_uuid")]
    pub uuid: Uuid,
    pub bin: String,
    pub parameters: Vec<JobConfCommandParam>
}

#[cfg(not(tarpaulin_include))]
fn get_uuid() -> Uuid {
    Uuid::new_v4()
}

#[cfg(not(tarpaulin_include))]
#[derive(Debug, PartialEq, Eq)]
pub enum JobConfError {
    ParseIntError,
    ParseBoolError,
    ParseFloatError,
    MissingValue,
}

#[cfg(not(tarpaulin_include))]
impl JobConfError{
    fn get_description(&self) -> &str {
        match &self {
            JobConfError::ParseIntError => "encountered error while parsing default value to int",
            JobConfError::ParseBoolError => "encountered error while parsing default value to bool",
            JobConfError::ParseFloatError => "encountered error while parsing default value to float",
            JobConfError::MissingValue => "a field value is missing",
        }
    }
}

#[cfg(not(tarpaulin_include))]
impl fmt::Display for JobConfError {
    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
        write!( formatter, "{}", self.get_description())
    }
}

#[cfg(not(tarpaulin_include))]
impl Error for JobConfError {}

#[derive(Debug, PartialEq, Eq)]
pub enum CommandError {
    MissingRequiredParam,
}

#[cfg(not(tarpaulin_include))]
impl CommandError {
    fn get_description(&self) -> &str {
        match &self {
            CommandError::MissingRequiredParam => "missing required parameter in the given parameters",
        }
    }
}

#[cfg(not(tarpaulin_include))]
impl fmt::Display for CommandError {
    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
        write!( formatter, "{}", self.get_description())
    }
}

#[derive(Debug, PartialEq, Eq)]
pub enum AdapterErrorKind {
    ParseError,
    IOError
}

#[cfg(not(tarpaulin_include))]
impl AdapterErrorKind{
    fn get_description(&self) -> &str {
        match &self {
            AdapterErrorKind::ParseError => "an error happened during the configuration file's syntax checking (see the associated error for more details)",
            AdapterErrorKind::IOError => "an error happened during the configuration file io manipulation (see the associated error for more details)",
        }
    }
}

#[derive(Debug)]
pub struct AdapterError {
    pub kind: AdapterErrorKind,
    pub error: Option<Box<dyn Error + 'static>>
}

#[cfg(not(tarpaulin_include))]
impl fmt::Display for AdapterError {
    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
        write!( formatter, "{}", self.kind.get_description())
    }
}

pub type Configurations = HashMap<String, HashMap<String, HashMap<String, JobConfCommand>>>;

#[derive(Clone, Debug, PartialEq, Eq)]
pub struct SlurmJobInfos {
    pub run_time: String,
    pub submit_time: String,
    pub start_time: String,
    pub end_time: String,
    pub status: String
}
