use std::env;
use std::io::Write;
use std::path::Path;
use std::fs::File;
use uuid::Uuid;
use crate::constants::MiddlewareError;
use dto::jobs::*;
use model::establish_connection;
use model::models::jobs::NewJob;
use adapters::constants::{Configurations, SBatch};
use adapters::adapters::{ generate_command, generate_sbatch, run_sbatch };

pub fn insert(data: CreateJob, config: Configurations) -> Result<Job, MiddlewareError> {
    let mut connection = establish_connection();
    // Getting the default parameter of the job in the appropriate config file
    let conf = match config.get(&data.job.software) {
        Some(software) => {
            match software.get(&data.job.version) {
                Some(version) => {
                    match version.get(&data.job.job_type) {
                        Some(val) => val,
                        None => return Err(MiddlewareError::ConfigError("".to_string()))
                    }
                },
                None =>return Err(MiddlewareError::ConfigError("".to_string()))
            }
        },
        None => return Err(MiddlewareError::ConfigError("".to_string()))
    };
    // Generate the task command
    let command = match generate_command(data.params, conf.clone()) {
        Ok(val) => val,
        Err(err) => return Err(MiddlewareError::CommandError(err))
    };
    // Generate the slurm batch script that will launch the task
    let sbatch = match generate_sbatch(data.hardware, command) {
        Ok(val) => val,
        Err(err) =>  return Err(MiddlewareError::GenerateSbatchError(err))
    };
    // Getting sbatch directory set as a en env var
    let sbtach_dir = match env::var("SBATCH_FOLDER") {
        Ok(val) => val,
        Err(err) => return Err(MiddlewareError::EnVarError(err))
    };
    // Create an sbatch object to easily create it's save path
    let sbatch_object = SBatch::new(data.job.job_type.clone(), data.job.version.clone(), data.job.software.clone());
    // Combine the base path and the path given for the previously created object to get
    // the script final path on the filesystem
    let final_path = Path::new(&sbtach_dir).join(sbatch_object.get_dir());
    // Create the output file
    let mut output_file = match File::create(&final_path) {
        Ok(val) => val,
        Err(err) => return Err(MiddlewareError::SbatchError(err))
    };
    // Save the slurm batch script to the filesystem
    match output_file.write_all(sbatch.as_bytes()) {
        Ok(_) => (),
        Err(err) => return Err(MiddlewareError::SbatchError(err))
    }
    // Run the generated sbatch
    let slurm_job_id = match run_sbatch(&final_path) {
        Ok(val) => val,
        Err(err) => return Err(MiddlewareError::SbatchError(err))
    };
    // Trasnform the received data in a diesel compatible struct
    let mut new_job = match NewJob::try_from(data.job) {
        Ok(job) => job,
        Err(err) => return Err(MiddlewareError::ParseError(err))
    };
    // Add to the new structure the slurm job id
    new_job.slurm_id = slurm_job_id;
    // Insert this new struct in the database with Diesel
    match model::operations::jobs::insert(&mut connection, new_job) {
        Ok(job) => Ok(job.into()),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn find_by_id(id: Uuid) -> Result<Option<Job>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::jobs::find_by_id(&mut connection, id) {
        Ok(job_option) => match job_option {
            Some(found) => Ok(Some(found.into())),
            None => Ok(None)
        },
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn update(id: Uuid, job: dto::jobs::UpdateJob) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::jobs::update(&mut connection, id, job.into()) {
        Ok(job) => Ok(job),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn delete(id: Uuid) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::jobs::delete(&mut connection, id) {
        Ok(job) => Ok(job),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn find_by_jobgroup(searched_group_id: uuid::Uuid) -> Result<Vec<(Job, i32)>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::jobs::get_all_for_group(&mut connection, searched_group_id) {
        Ok(jobs) => {
            let mut job_list = Vec::new();
            for job in jobs {
                job_list.push((job.0.into(), job.1));
            }
            Ok(job_list)
        }
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}
