use crate::auth::Authorization;
use crate::constants::ErrorMessages;
use rocket::http::Status;
use rocket::response::status;
use rocket::serde::{json::Json, uuid::Uuid};

use dto::jobgroups::*;
use dto::jobs::Job;
use dto::nodes::Node;

use middleware::models::jobs::find_by_jobgroup as find_job_by_jobgroup;
use middleware::models::nodes::get_all_for_group as find_nodes_by_jobgroup;

use rocket_okapi::openapi;

/// # Creates a new job group
///
/// Creates a new job group and returns it
///
/// Returns status code 500 if it failed
///
/// Returns status code 201 if it succeeded, with the new ressource created in the location header
#[openapi]
#[post("/", data = "<jobgroup>")]
pub fn create_jobgroup(
    jobgroup: Json<InsertJobGroup>,
    _auth: Authorization
) -> Result<status::Created<&'static str>, (Status, String)> {
    match middleware::models::jobgroups::insert(jobgroup.into_inner()) {
        Ok(created_group) => Ok(status::Created::new(created_group.id.to_string())),
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Returns the job group identified by its id
///
/// Returns status code 500 if it failed
///
/// Returns status code 404 if the job group was not found
///
/// Returns status code 200 the user identified by its id
#[openapi]
#[get("/<id>")]
pub fn get_jobgroup(id: Uuid,_auth: Authorization) -> Result<Json<JobGroup>, (Status, String)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::jobgroups::find_by_id(parsed_id) {
        Ok(option_found_jobgroup) => match option_found_jobgroup {
            Some(found_jobgroup) => Ok(Json(found_jobgroup)),
            None => Err((Status::NotFound, ErrorMessages::IdNotFound.to_string()))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Updates the job group with the informations given
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 if the update succeeded
#[openapi]
#[put("/<id>", format = "json", data = "<jobgroup>")]
pub fn update_jobgroup(
    id: Uuid,
    jobgroup: Json<dto::jobgroups::UpdateJobGroup>,
    _auth: Authorization
) -> Result<Status, (Status, Option<String>)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    let updated_jobgroup_count =
        middleware::models::jobgroups::update(parsed_id, jobgroup.into_inner());
    match updated_jobgroup_count {
        Ok(_) => Ok(Status::Ok),
        Err(error) => Err((Status::InternalServerError, Some(error.to_string())))
    }
}

/// # Deletes an user identified by its id
///
/// Returns status code 404 if the id does not exist
///
/// Returns status code 500 if it failed for some other reason
///
/// Returns status code 200 if it succeeded, with the number of users deleted
#[openapi]
#[delete("/<id>")]
pub fn delete_jobgroup(id: Uuid, _auth: Authorization) -> Result<(Status, String), (Status, String)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::jobgroups::delete(parsed_id) {
        Ok(deleted_rows_count) => match deleted_rows_count {
            0 => Err((Status::NotFound, "No row deleted".to_string())),
            _ => Ok((Status::Ok, format!("{} row deleted", deleted_rows_count)))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Returns the jobs belonging to a job group
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 and the list of jobs for the specified job group if it succeeded
#[openapi]
#[get("/<group_id>/jobs")]
pub fn get_jobs(group_id: Uuid, _auth: Authorization) -> Result<Json<Vec<Job>>, (Status, String)> {
    let parsed_group_id = uuid::Uuid::parse_str(group_id.to_string().as_str()).unwrap();

    match find_job_by_jobgroup(parsed_group_id) {
        Ok(found_jobs) => {
            let mut final_list: Vec<Job> = Vec::new();
            for e in found_jobs.into_iter() {
                final_list.push(e.0);
            }
            Ok(Json(final_list))
        }
        Err(err) => Err((Status::InternalServerError, err.to_string()))
    }
}

/// # Returns the nodes belonging to a job group
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 and the list of nodes for the specified job group if it succeeded
#[openapi]
#[get("/<jobgroup_id>/nodes")]
pub fn get_nodes(
    jobgroup_id: Uuid,
    _auth: Authorization
) -> Result<Json<Vec<Node>>, (Status, String)> {
    let parsed_jobgroup_id = uuid::Uuid::parse_str(jobgroup_id.to_string().as_str()).unwrap();
    match find_nodes_by_jobgroup(parsed_jobgroup_id) {
        Ok(found_nodes) => Ok(Json(found_nodes)),
        Err(err) => Err((Status::InternalServerError, err.to_string()))
    }
}
