use crate::auth::Authorization;
use crate::constants::ErrorMessages;
use middleware::models::parameters::find_by_job;
use rocket::State;
use rocket::http::Status;
use rocket::response::status;
use rocket::serde::{json::Json, uuid::Uuid};

use dto::jobs::*;
use dto::parameters::GetParam;

use adapters::constants::Configurations;

use rocket_okapi::openapi;

/// # Creates a new job
///
/// Creates a new job and returns it
///
/// Returns status code 500 if it failed
///
/// Returns status code 201 if it succeeded, with the new ressource created in the location header
#[openapi]
#[post("/", data = "<job>")]
pub fn create_job(
    job: Json<CreateJob>,
    config: &State<Configurations>,
    _auth: Authorization
) -> Result<status::Created<&'static str>, (Status, String)> {
    match middleware::models::jobs::insert(job.into_inner(), config.inner().clone()) {
        Ok(created_job) => Ok(status::Created::new(created_job.id.to_string())),
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Returns the job identified by its id
///
/// Returns status code 500 if it failed
///
/// Returns status code 404 if the job was not found
///
/// Returns status code 200 the job identified by its id
#[openapi]
#[get("/<id>")]
pub fn get_job(id: Uuid, _auth: Authorization) -> Result<Json<Job>, (Status, String)> {
    let parsed_job_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::jobs::find_by_id(parsed_job_id) {
        Ok(option_found_job) => match option_found_job {
            Some(found_job) => Ok(Json(found_job)),
            None => Err((Status::NotFound, ErrorMessages::IdNotFound.to_string()))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Updates the job with the informations given
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 if the update succeeded
#[openapi]
#[put("/<id>", format = "json", data = "<job>")]
pub fn update_job(
    id: Uuid,
    job: Json<dto::jobs::UpdateJob>,
    _auth: Authorization
) -> Result<Status, (Status, Option<String>)> {
    let parsed_job_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    let updated_job_count = middleware::models::jobs::update(parsed_job_id, job.into_inner());
    match updated_job_count {
        Ok(_) => Ok(Status::Ok),
        Err(error) => Err((Status::InternalServerError, Some(error.to_string())))
    }
}

/// # Deletes a job identified by its id
///
/// Returns status code 404 if the id does not exist
///
/// Returns status code 500 if it failed for some other reason
///
/// Returns status code 200 if it succeeded, with the number of jobs deleted
#[openapi]
#[delete("/<id>")]
pub fn delete_job(id: Uuid, _auth: Authorization) -> Result<(Status, String), (Status, String)> {
    let parsed_job_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::jobs::delete(parsed_job_id) {
        Ok(deleted_rows_count) => match deleted_rows_count {
            0 => Err((Status::NotFound, "No row deleted".to_string())),
            _ => Ok((Status::Ok, format!("{} row deleted", deleted_rows_count)))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Returns the parameters belonging to a job
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 and the list of parameters for the specified job if it succeeded
#[openapi]
#[get("/<job_id>/parameters")]
pub fn get_parameters(
    job_id: Uuid,
    _auth: Authorization
) -> Result<Json<Vec<GetParam>>, (Status, String)> {
    let parsed_job_id = uuid::Uuid::parse_str(job_id.to_string().as_str()).unwrap();
    match find_by_job(parsed_job_id) {
        Ok(found_params) => {
            let mut final_list: Vec<GetParam> = Vec::new();
            for e in found_params.into_iter() {
                let result = GetParam {
                    id: e.id,
                    job_id: e.job_id,
                    name: e.name,
                    value: e.value
                };
                final_list.push(result);
            }
            Ok(Json(final_list))
        }
        Err(err) => Err((Status::InternalServerError, err.to_string()))
    }
}
