use crate::auth::Authorization;
use crate::constants::ErrorMessages;
use rocket::http::Status;
use rocket::response::status;
use rocket::serde::{json::Json, uuid::Uuid};

use dto::nodes::*;

use rocket_okapi::openapi;

/// # Creates a new node
///
/// Creates a new node and returns it
///
/// Returns status code 500 if it failed
///
/// Returns status code 201 if it succeeded, with the new ressource created in the location header
#[openapi]
#[post("/", data = "<node>")]
pub fn create_node(
    node: Json<InsertNode>,
    _auth: Authorization
) -> Result<status::Created<&'static str>, (Status, String)> {
    match middleware::models::nodes::insert(node.into_inner()) {
        Ok(created_node) => Ok(status::Created::new(created_node.id.to_string())),
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Returns the node identified by its id
///
/// Returns status code 500 if it failed
///
/// Returns status code 404 if the node was not found
///
/// Returns status code 200 the user identified by its id
#[openapi]
#[get("/<id>")]
pub fn get_node(id: Uuid, _auth: Authorization) -> Result<Json<Node>, (Status, String)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::nodes::find_by_id(parsed_id) {
        Ok(option_found_node) => match option_found_node {
            Some(found_node) => Ok(Json(found_node)),
            None => Err((Status::NotFound, ErrorMessages::IdNotFound.to_string()))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Updates the node with the informations given
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 if the update succeeded
#[openapi]
#[put("/<id>", format = "json", data = "<node>")]
pub fn update_node(
    id: Uuid,
    node: Json<dto::nodes::UpdateNode>,
    _auth: Authorization
) -> Result<Status, (Status, Option<String>)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    let updated_node_count = middleware::models::nodes::update(parsed_id, node.into_inner());
    match updated_node_count {
        Ok(_) => Ok(Status::Ok),
        Err(error) => Err((Status::InternalServerError, Some(error.to_string())))
    }
}

/// # Deletes a node identified by its id
///
/// Returns status code 404 if the id does not exist
///
/// Returns status code 500 if it failed for some other reason
///
/// Returns status code 200 if it succeeded, with the number of nodes deleted
#[openapi]
#[delete("/<id>")]
pub fn delete_node(id: Uuid, _auth: Authorization) -> Result<(Status, String), (Status, String)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::nodes::delete(parsed_id) {
        Ok(deleted_rows_count) => match deleted_rows_count {
            0 => Err((Status::NotFound, "No row deleted".to_string())),
            _ => Ok((Status::Ok, format!("{} row deleted", deleted_rows_count)))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

// https://www.restapitutorial.com/lessons/httpmethods.html
