use crate::auth::Authorization;
use crate::constants::ErrorMessages;
use rocket::http::Status;
use rocket::response::status;
use rocket::serde::{json::Json, uuid::Uuid};

use dto::parameters::*;

use rocket_okapi::openapi;

/// # Creates a new parameter
///
/// Creates a new parameter and returns it
///
/// Returns status code 500 if it failed
///
/// Returns status code 201 if it succeeded, with the new ressource created in the location header
#[openapi]
#[post("/", data = "<parameter>")]
pub fn create_parameter(
    parameter: Json<InsertParam>,
    _auth: Authorization
) -> Result<status::Created<&'static str>, (Status, String)> {
    match middleware::models::parameters::insert(parameter.into_inner()) {
        Ok(created_param) => Ok(status::Created::new(created_param.id.to_string())),
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Returns the parameter identified by its id
///
/// Returns status code 500 if it failed
///
/// Returns status code 404 if the parameter was not found
///
/// Returns status code 200 the parameter identified by its id
#[openapi]
#[get("/<id>")]
pub fn get_parameter(id: Uuid, _auth: Authorization) -> Result<Json<GetParam>, (Status, String)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::parameters::find_by_id(parsed_id) {
        Ok(option_found_param) => match option_found_param {
            Some(found_param) => {
                let result = GetParam {
                    id: found_param.id,
                    job_id: found_param.job_id,
                    name: found_param.name,
                    value: found_param.value
                };
                Ok(Json(result))
            }
            None => Err((Status::NotFound, ErrorMessages::IdNotFound.to_string()))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Updates the parameter with the informations given
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 if the update succeeded
#[openapi]
#[put("/<id>", format = "json", data = "<param>")]
pub fn update_param(
    id: Uuid,
    param: Json<dto::parameters::UpdateParam>,
    _auth: Authorization
) -> Result<Status, (Status, Option<String>)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    let updated_param_count = middleware::models::parameters::update(parsed_id, param.into_inner());
    match updated_param_count {
        Ok(_) => Ok(Status::Ok),
        Err(error) => Err((Status::InternalServerError, Some(error.to_string())))
    }
}

/// # Deletes a parameter identified by its id
///
/// Returns status code 404 if the id does not exist
///
/// Returns status code 500 if it failed for some other reason
///
/// Returns status code 200 if it succeeded, with the number of parameters
#[openapi]
#[delete("/<id>")]
pub fn delete_param(id: Uuid, _auth: Authorization) -> Result<(Status, String), (Status, String)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::parameters::delete(parsed_id) {
        Ok(deleted_rows_count) => match deleted_rows_count {
            0 => Err((Status::NotFound, "No row deleted".to_string())),
            _ => Ok((Status::Ok, format!("{} row deleted", deleted_rows_count)))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

// https://www.restapitutorial.com/lessons/httpmethods.html
