// Disable clippy warning for option routes
#![allow(clippy::let_unit_value)]
use crate::auth::Authorization;
use crate::constants::ErrorMessages;
use dto::workspaces::Workspace;
use rocket::http::Status;
use rocket::response::status;
use rocket::serde::{json::Json, uuid::Uuid};

use dto::projects::*;
use middleware::models::workspaces::find_by_project;

use rocket_okapi::openapi;

/// # Creates a new project
///
/// Creates a new project and returns it
///
/// Returns status code 500 if it failed
///
/// Returns status code 201 if it succeeded, with the new ressource created in the location header
#[openapi]
#[post("/", data = "<project>")]
pub fn create_project(
    project: Json<InsertProject>,
    _auth: Authorization
) -> Result<status::Created<&'static str>, (Status, String)> {
    match middleware::models::projects::insert(project.into_inner()) {
        Ok(created_project) => Ok(status::Created::new(created_project.id.to_string())),
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Returns the project identified by its id
///
/// Returns status code 500 if it failed
///
/// Returns status code 404 if the project was not found
///
/// Returns status code 200 the project identified by its id
#[openapi]
#[get("/<id>")]
pub fn get_project(id: Uuid, _auth: Authorization) -> Result<Json<Project>, (Status, String)> {
    let parsed_project_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::projects::find_by_id(parsed_project_id) {
        Ok(option_found_project) => match option_found_project {
            Some(found_project) => Ok(Json(found_project)),
            None => Err((Status::NotFound, ErrorMessages::IdNotFound.to_string()))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Updates the project with the informations given
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 if the update succeeded
#[openapi]
#[put("/<id>", format = "json", data = "<project>")]
pub fn update_project(
    id: Uuid,
    project: Json<dto::projects::UpdateProject>,
    _auth: Authorization
) -> Result<Status, (Status, Option<String>)> {
    let parsed_project_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    let updated_project_count =
        middleware::models::projects::update(parsed_project_id, project.into_inner());
    match updated_project_count {
        Ok(_) => Ok(Status::Ok),
        Err(error) => Err((Status::InternalServerError, Some(error.to_string())))
    }
}

/// # Deletes an user identified by its id
///
/// Returns status code 404 if the id does not exist
///
/// Returns status code 500 if it failed for some other reason
///
/// Returns status code 200 if it succeeded, with the number of users
#[openapi]
#[delete("/<id>")]
pub fn delete_project(id: Uuid, _auth: Authorization) -> Result<(Status, String), (Status, String)> {
    let parsed_project_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::projects::delete(parsed_project_id) {
        Ok(deleted_rows_count) => match deleted_rows_count {
            0 => Err((Status::NotFound, "No row deleted".to_string())),
            _ => Ok((Status::Ok, format!("{} row deleted", deleted_rows_count)))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Returns the workspaces belonging to a project
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 and the list of projects for the specified user if it succeeded
#[openapi]
#[get("/<project_id>/workspaces")]
pub fn get_workspaces(
    project_id: Uuid,
    _auth: Authorization
) -> Result<Json<Vec<Workspace>>, (Status, String)> {
    let parsed_project_id = uuid::Uuid::parse_str(project_id.to_string().as_str()).unwrap();
    match find_by_project(parsed_project_id) {
        Ok(found_workspaces) => Ok(Json(found_workspaces)),
        Err(err) => Err((Status::InternalServerError, err.to_string()))
    }
}

#[openapi]
#[options("/")]
pub fn option_projects() {}

#[openapi]
#[options("/<_id>")]
pub fn option_get_project(_id: Uuid) {}

// https://www.restapitutorial.com/lessons/httpmethods.html
