use crate::auth::Authorization;
use rocket::http::Status;
use rocket::serde::json::Json;

use dto::projects::Project;
use dto::users::User;

use middleware::models::projects::find_by_team as find_projects_by_team;
use middleware::models::users::find_by_team as find_users_by_team;

use rocket_okapi::openapi;

/// # Returns the list of users in a specific team
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 if it succeeded, with the list of users
#[openapi]
#[get("/<team>/users")]
pub fn get_user_in_team(
    team: String,
    _auth: Authorization
) -> Result<Json<Vec<User>>, (Status, String)> {
    match find_users_by_team(team) {
        Ok(found_users) => {
            let mut final_list: Vec<User> = Vec::new();
            for e in found_users.into_iter() {
                final_list.push(e);
            }
            Ok(Json(final_list))
        }
        Err(err) => Err((Status::InternalServerError, err.to_string()))
    }
}

/// # Returns the list of projects for the specified team
///
/// Returns status code 200 if it succeeded, with the list of projects belonging to a specific team
#[openapi]
#[get("/<team>/projects")]
pub fn get_team_project_list(
    team: String,
    _auth: Authorization
) -> Result<Json<Vec<Project>>, (Status, String)> {
    match find_projects_by_team(team) {
        Ok(found_projects) => Ok(Json(found_projects)),
        Err(err) => Err((Status::InternalServerError, err.to_string()))
    }
}
