use crate::auth::Authorization;
use crate::constants::ErrorMessages;
use dto::projects::Project;
use rocket::http::Status;
use rocket::response::status;
use rocket::serde::{json::Json, uuid::Uuid};

use dto::users::*;

use rocket_okapi::openapi;

/// # Creates a new user
///
/// Creates a new user and returns it
///
/// Returns status code 500 if it failed
///
/// Returns status code 201 if it succeeded, with the new ressource created in the location header
#[openapi]
#[post("/", data = "<user>")]
pub fn create_user(
    user: Json<InsertUser>,
    _auth: Authorization
) -> Result<status::Created<&'static str>, (Status, String)> {
    match middleware::models::users::insert(user.into_inner()) {
        Ok(created_user) => Ok(status::Created::new(created_user.id.to_string())),
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Returns the list of users
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 and the list of all users
#[openapi]
#[get("/")]
pub fn get_user_list(_auth: Authorization) -> Result<Json<Vec<User>>, (Status, String)> {
    match middleware::models::users::list() {
        Ok(found_users) => Ok(Json(found_users)),
        Err(err) => Err((Status::InternalServerError, err.to_string()))
    }
}

/// # Returns the projects belonging to a user
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 and the list of projects for the specified user if it succeeded
#[openapi]
#[get("/<user_id>/projects")]
pub fn get_projects(
    user_id: Uuid,
    _auth: Authorization
) -> Result<Json<Vec<Project>>, (Status, String)> {
    let parsed_user_id = uuid::Uuid::parse_str(user_id.to_string().as_str()).unwrap();
    match middleware::models::projects::find_by_user(parsed_user_id) {
        Ok(found_projects) => Ok(Json(found_projects)),
        Err(err) => Err((Status::InternalServerError, err.to_string()))
    }
}

/// # Returns the user identified by its id
///
/// Returns status code 500 if it failed
///
/// Returns status code 404 if the user was not found
///
/// Returns status code 200 the user identified by its id
#[openapi]
#[get("/<id>")]
pub fn get_user(id: Uuid, _auth: Authorization) -> Result<Json<User>, (Status, String)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::users::find_by_id(parsed_id) {
        Ok(option_found_user) => match option_found_user {
            Some(found_user) => Ok(Json(found_user)),
            None => Err((Status::NotFound, ErrorMessages::IdNotFound.to_string()))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Updates the user with the informations given
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 if the update succeeded
#[openapi]
#[put("/<id>", format = "json", data = "<user>")]
pub fn update_user(
    id: Uuid,
    user: Json<dto::users::UpdateUser>,
    _auth: Authorization
) -> Result<Status, (Status, Option<String>)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    let updated_user_count = middleware::models::users::update(parsed_id, user.into_inner());
    match updated_user_count {
        Ok(_) => Ok(Status::Ok),
        Err(error) => Err((Status::InternalServerError, Some(error.to_string())))
    }
}

/// # Deletes an user identified by its id
///
/// Returns status code 404 if the id does not exist
///
/// Returns status code 500 if it failed for some other reason
///
/// Returns status code 200 if it succeeded, with the number of users deleted
#[openapi]
#[delete("/<id>")]
pub fn delete_user(id: Uuid, _auth: Authorization) -> Result<(Status, String), (Status, String)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::users::delete(parsed_id) {
        Ok(deleted_rows_count) => match deleted_rows_count {
            0 => Err((Status::NotFound, "No row deleted".to_string())),
            _ => Ok((Status::Ok, format!("{} row deleted", deleted_rows_count)))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

// https://www.restapitutorial.com/lessons/httpmethods.html
