use crate::auth::Authorization;
use crate::constants::ErrorMessages;
use rocket::http::Status;
use rocket::response::status;
use rocket::serde::{json::Json, uuid::Uuid};

use dto::jobgroups::JobGroup;
use dto::workspaces::*;

use rocket_okapi::openapi;

/// # Creates a new workspace
///
/// Creates a new workspace and returns it
///
/// Returns status code 500 if it failed
///
/// Returns status code 201 if it succeeded, with the new ressource created in the location header
#[openapi]
#[post("/", data = "<workspace>")]
pub fn create_workspace(
    workspace: Json<InsertWorkspace>,
    _auth: Authorization
) -> Result<status::Created<&'static str>, (Status, String)> {
    match middleware::models::workspaces::insert(workspace.into_inner()) {
        Ok(created_workspace) => Ok(status::Created::new(created_workspace.id.to_string())),
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Returns the workspace identified by its id
///
/// Returns status code 500 if it failed
///
/// Returns status code 404 if the user was not found
///
/// Returns status code 200 the user identified by its id
#[openapi]
#[get("/<id>")]
pub fn get_workspace(id: Uuid, _auth: Authorization) -> Result<Json<GetWorkspace>, (Status, String)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::workspaces::find_by_id(parsed_id) {
        Ok(option_found_workspace) => match option_found_workspace {
            Some(found_workspace) => {
                let result = GetWorkspace {
                    id: found_workspace.id,
                    name: found_workspace.name,
                    project_id: found_workspace.project_id
                };
                Ok(Json(result))
            }
            None => Err((Status::NotFound, ErrorMessages::IdNotFound.to_string()))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Updates the workspace with the informations given
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 if the update succeeded
#[openapi]
#[put("/<id>", format = "json", data = "<workspace>")]
pub fn update_workspace(
    id: Uuid,
    workspace: Json<dto::workspaces::UpdateWorkspace>,
    _auth: Authorization
) -> Result<Status, (Status, Option<String>)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    let updated_workspace_count =
        middleware::models::workspaces::update(parsed_id, workspace.into_inner());
    match updated_workspace_count {
        Ok(_) => Ok(Status::Ok),
        Err(error) => Err((Status::InternalServerError, Some(error.to_string())))
    }
}

/// # Deletes a workspace identified by its id
///
/// Returns status code 404 if the id does not exist
///
/// Returns status code 500 if it failed for some other reason
///
/// Returns status code 200 if it succeeded, with the number of workspaces deleted
#[openapi]
#[delete("/<id>")]
pub fn delete_workspace(
    id: Uuid,
    _auth: Authorization
) -> Result<(Status, String), (Status, String)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match middleware::models::workspaces::delete(parsed_id) {
        Ok(deleted_rows_count) => match deleted_rows_count {
            0 => Err((Status::NotFound, "No row deleted".to_string())),
            _ => Ok((Status::Ok, format!("{} row deleted", deleted_rows_count)))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

/// # Returns the list of jobgroup for the specified workspace
///
/// Returns status code 500 if it failed for some other reason
///
/// Returns status code 200 if it succeeded, with the list of jobgroups
#[openapi]
#[get("/<workspace_id>/groups")]
pub fn get_groups(
    workspace_id: Uuid,
    _auth: Authorization
) -> Result<Json<Vec<JobGroup>>, (Status, String)> {
    let parsed_id = uuid::Uuid::parse_str(workspace_id.to_string().as_str()).unwrap();
    match middleware::models::jobgroups::find_by_workspace(parsed_id) {
        Ok(found_groups) => Ok(Json(found_groups)),
        Err(err) => Err((Status::InternalServerError, err.to_string()))
    }
}

// https://www.restapitutorial.com/lessons/httpmethods.html
