use jsonwebtoken::{encode, Algorithm, EncodingKey, Header};
use chrono::Duration;
use chrono::Utc;
use rocket::serde::uuid::Uuid;
use router::auth::{get_key, JWTClaims};

use model::establish_connection;
use model::models::users::NewUser;
use model::operations::users::{insert, delete};
use model::operations::projects::delete as delete_project_in_db;

pub fn initialize_env() {
    let path = std::env::current_dir().unwrap().into_os_string().into_string().unwrap() + "/tests/common/.env";
    println!("{}", path);
    dotenv::from_path(path).ok();
}

pub fn initialize_jwt_token() -> std::string::String {
    let now = Utc::now();
    let iat = now.timestamp();
    let exp = (now + Duration::days(1)).timestamp();
    let sub = Uuid::parse_str("f3a142eb-0fad-4eb8-b0d8-2d2dfdccff02").unwrap();
    let claims = JWTClaims { iat, exp, sub };
    let encoding_key = EncodingKey::from_rsa_pem(&get_key("JWT_PRIVATE_KEY_PATH")).unwrap();
    let header = Header::new(Algorithm::RS256);
    encode(&header, &claims, &encoding_key).unwrap()
}

pub fn create_user() -> Result<uuid::Uuid, String> {
    let mut connection = establish_connection();
    let new_user = NewUser {
        team: String::from("test"),
        identity: Some("f3a142eb-0fad-4eb8-b0d8-2d2dfdccff02".to_string())
    };
    match insert(&mut connection, new_user) {
        Ok(created_user) => Ok(created_user.id),
        Err(error) => Err(error.to_string()),
    }
}

pub fn delete_user(id: uuid::Uuid) -> Result<u32, String> {
    let mut connection = establish_connection();
    match delete(&mut connection, id) {
        Ok(deleted_rows_count) => match deleted_rows_count {
            0 => Err("No row deleted".to_string()),
            _ => Ok(deleted_rows_count.try_into().unwrap()),
        },
        Err(error) => Err(error.to_string()),
    }
}

pub fn delete_project(id: uuid::Uuid) -> Result<u32, String> {
    let mut connection = establish_connection();
    match delete_project_in_db(&mut connection, id) {
        Ok(deleted_rows_count) => match deleted_rows_count {
            0 => Err("No row deleted".to_string()),
            _ => Ok(deleted_rows_count.try_into().unwrap()),
        },
        Err(error) => Err(error.to_string()),
    }
}
