use rocket::http::Status;
use rocket::local::blocking::Client;
use rocket::routes;
use serde_json::json;

mod common;
use common::*;
use dto::projects::*;
use uuid::Uuid;

#[test]
fn test_api_create_project() {
    use router::routes::projects::create_project;
    
    initialize_env();
    let jwt_token = initialize_jwt_token();
    let user_id = match create_user() {
        Ok(val) => val,
        Err(_) => return ()
    };
    let rocket = rocket::build().mount("/projects", routes![create_project]);
    let client = Client::tracked(rocket).unwrap();
    let body = json!(InsertProject {
        name: String::from("test_api_create_project"),
        user_id: user_id
    });
    let auth_header = rocket::http::Header::new("Authorization", "Bearer ".to_owned() + &jwt_token);
    let response = client
        .post("/projects/")
        .body(body.to_string())
        .header(auth_header)
        .dispatch();
    println!("{:?}", response);
    assert_eq!(response.status(), Status::Created);
    assert!(response.body().is_none());
    assert!(response.headers().contains("Location"));
    delete_project(Uuid::parse_str(response.headers().get_one("Location").unwrap()).unwrap())
        .unwrap();
    delete_user(user_id).unwrap();
}

#[test]
fn test_api_get_project() {
    use router::routes::projects::{create_project, get_project};
    
    initialize_env();
    let jwt_token = initialize_jwt_token();
    let auth_header = rocket::http::Header::new("Authorization", "Bearer ".to_owned() + &jwt_token);
    let user_id = match create_user() {
        Ok(val) => val,
        Err(_) => return ()
    };
    let rocket = rocket::build().mount("/projects", routes![create_project, get_project]);
    let client = Client::tracked(rocket).unwrap();
    let project_name = String::from("test_api_get_project");
    let body = json!(InsertProject {
        name: project_name.clone(),
        user_id: user_id
    });
    let new_project_response = client
        .post("/projects/")
        .body(body.to_string())
        .header(auth_header.clone())
        .dispatch();
    let location_header = new_project_response.headers().get_one("Location").unwrap();
    let response = client
        .get(format!("/projects/{}", location_header))
        .header(auth_header.clone())
        .dispatch();
    assert_eq!(response.status(), Status::Ok);
    let json_response = response.into_json::<Project>().unwrap();
    assert_eq!(json_response.name, project_name);
    assert_eq!(json_response.id, Uuid::parse_str(location_header).unwrap());
    delete_project(
        Uuid::parse_str(new_project_response.headers().get_one("Location").unwrap()).unwrap(),
    )
    .unwrap();
    delete_user(user_id).unwrap();
}

#[test]
fn test_api_get_project_not_found() {
    use router::routes::projects::{create_project, get_project};

    initialize_env();
    let rocket = rocket::build().mount("/projects", routes![create_project, get_project]);
    let client = Client::tracked(rocket).unwrap();
    let jwt_token = initialize_jwt_token();
    let auth_header = rocket::http::Header::new("Authorization", "Bearer ".to_owned() + &jwt_token);
    let response = client.get("/projects/0").header(auth_header).dispatch();
    assert_eq!(response.status(), Status::NotFound);
}

#[test]
fn test_api_update_project() {
    use rocket::http::ContentType;
    use router::routes::projects::{create_project, get_project, update_project};

    initialize_env();
    let rocket = rocket::build().mount(
        "/projects",
        routes![create_project, get_project, update_project],
    );
    let client = Client::tracked(rocket).unwrap();
    let user_id = match create_user() {
        Ok(val) => val,
        Err(_) => return ()
    };
    let updated_name = String::from("test_api_update_project_updated_name");
    let body = json!(InsertProject {
        name: String::from("test_api_update_project"),
        user_id: user_id
    });
    let updated_body = json!(InsertProject {
        name: updated_name.clone(),
        user_id: user_id
    });
    let jwt_token = initialize_jwt_token();
    let auth_header = rocket::http::Header::new("Authorization", "Bearer ".to_owned() + &jwt_token);
    let new_project_response = client
        .post("/projects/")
        .body(body.to_string())
        .header(auth_header.clone())
        .dispatch();
    let location_header = new_project_response.headers().get_one("Location").unwrap();
    let update_response = client
        .put(format!("/projects/{}", location_header))
        .body(updated_body.to_string())
        .header(ContentType::JSON)
        .header(auth_header.clone())
        .dispatch();
    assert_eq!(update_response.status(), Status::Ok);
    let get_response_json = client
        .get(format!("/projects/{}", location_header))
        .header(auth_header.clone())
        .dispatch()
        .into_json::<Project>()
        .unwrap();
    assert_eq!(get_response_json.name, updated_name);
    delete_project(
        uuid::Uuid::parse_str(new_project_response.headers().get_one("Location").unwrap()).unwrap(),
    )
    .unwrap();
    delete_user(user_id).unwrap();
}

#[test]
fn test_api_update_project_id_not_found() {
    use rocket::http::ContentType;
    use router::routes::projects::{create_project, get_project, update_project};

    initialize_env();
    let rocket = rocket::build().mount(
        "/projects",
        routes![create_project, get_project, update_project],
    );
    let client = Client::tracked(rocket).unwrap();
    let user_id = match create_user() {
        Ok(val) => val,
        Err(_) => return ()
    };
    let body = json!(InsertProject {
        name: String::from("test_api_update_project_not_found"),
        user_id: user_id
    });
    let jwt_token = initialize_jwt_token();
    let authorization_header = rocket::http::Header::new("Authorization", "Bearer ".to_owned() + &jwt_token);
    let response = client
        .put("/projects/0")
        .body(body.to_string())
        .header(ContentType::JSON)
        .header(authorization_header)
        .dispatch();
    assert_eq!(response.status(), Status::NotFound);
    delete_user(user_id).unwrap();
}

#[test]
fn test_api_update_project_without_content() {
    use rocket::http::ContentType;
    use router::routes::projects::{create_project, get_project, update_project};

    initialize_env();
    let rocket = rocket::build().mount(
        "/projects",
        routes![create_project, get_project, update_project],
    );
    let client = Client::tracked(rocket).unwrap();
    let user_id = match create_user() {
        Ok(val) => val,
        Err(_) => return ()
    };
    let body = json!(InsertProject {
        name: String::from("test_api_update_project"),
        user_id: user_id
    });
    let jwt_token = initialize_jwt_token();
    let auth_header = rocket::http::Header::new("Authorization", "Bearer ".to_owned() + &jwt_token);
    let new_project_response = client
        .post("/projects/")
        .body(body.to_string())
        .header(auth_header.clone())
        .dispatch();
    let location_header = new_project_response.headers().get_one("Location").unwrap();
    let update_response = client
        .put(format!("/projects/{}", location_header))
        .body("{}")
        .header(ContentType::JSON)
        .header(auth_header.clone())
        .dispatch();
    assert_eq!(update_response.status(), Status::InternalServerError);
    delete_project(uuid::Uuid::parse_str(location_header).unwrap()).unwrap();
    delete_user(user_id).unwrap();
}

#[test]
fn test_api_delete_project() {
    use router::routes::projects::{create_project, delete_project, get_project};

    initialize_env();
    let rocket = rocket::build().mount(
        "/projects",
        routes![create_project, get_project, delete_project],
    );
    let client = Client::tracked(rocket).unwrap();
    let user_id = match create_user() {
        Ok(val) => val,
        Err(_) => return ()
    };
    let body = json!(InsertProject {
        name: String::from("test_api_delete_project"),
        user_id: user_id
    });
    let jwt_token = initialize_jwt_token();
    let auth_header = rocket::http::Header::new("Authorization", "Bearer ".to_owned() + &jwt_token);
    let new_project = client
        .post("/projects/")
        .body(body.to_string())
        .header(auth_header.clone())
        .dispatch();
    let location_header = new_project.headers().get_one("Location").unwrap();
    let get_response = client
        .get(format!("/projects/{}", location_header))
        .header(auth_header.clone())
        .dispatch();
    assert_eq!(get_response.status(), Status::Ok);
    let delete_response = client
        .delete(format!("/projects/{}", location_header))
        .header(auth_header.clone())
        .dispatch()
        .into_string()
        .unwrap();
    assert_eq!("1 row deleted".to_string(), delete_response);
    let response_after_delete = client
        .get(format!("/projects/{}", location_header))
        .header(auth_header.clone())
        .dispatch();
    assert_eq!(response_after_delete.status(), Status::NotFound);
    delete_user(user_id).unwrap();
}

#[test]
fn test_api_delete_non_existant_project() {
    use router::routes::projects::delete_project;

    initialize_env();
    let rocket = rocket::build().mount("/projects", routes![delete_project]);
    let client = Client::tracked(rocket).unwrap();
    let jwt_token = initialize_jwt_token();
    let delete_auth_header = rocket::http::Header::new("Authorization", "Bearer ".to_owned() + &jwt_token);
    let delete_response = client
        .delete("/projects/00000000-0000-0000-0000-000000000000")
        .header(delete_auth_header)
        .dispatch()
        .into_string()
        .unwrap();
    assert_eq!("No row deleted".to_string(), delete_response);
}
